<?php
/*
Plugin Name: I18N Base
Description: Internationalization based on slug/URL names (e.g. index, index_de, index_fr)
Version: 0.9.7
Author: Martin Vlcek
Author URI: http://mvlcek.bplaced.net

Public functions:
  return_i18n_languages()
      returns an array of user requested languages with the best first, e.g. ( 'de', 'fr', 'en' )
  return_i18n_page_data($slug)
      returns the xml data for the best fitting language version of the $slug

Display functions:
  get_i18n_content($slug)
      outputs the best fitting language content of the $slug
  get_i18n_component($id)
      outputs the (localized) component
*/

# get correct id for plugin
$thisfile = basename(__FILE__, ".php");
$i18n_languages = null;
$i18n_settings = null;

# --- CHANGE THESE IF NEEDED ---
define('I18N_DEFAULT_LANGUAGE', 'en');
# --- CHANGE THESE END ---------

define('I18N_SETTINGS_FILE', 'i18n_settings.xml');
define('I18N_LANGUAGE_PARAM', 'lang');            # use to override language in URL, e.g. "...?lang=de"
define('I18N_SET_LANGUAGE_PARAM', 'setlang');     # use to set current language via GET/POST, e.g. "...?setlang=de"
define('I18N_LANGUAGE_KEY', 'language');          # session key for language, if language was set 

// properties
define('I18N_PROP_FILES_TO_IGNORE', 'files-to-ignore');
define('I18N_PROP_PAGES_VIEW', 'pages-view');

# register plugin
register_plugin(
	$thisfile, 
	'I18N Base', 	
	'0.9.7', 		
	'Martin Vlcek',
	'http://mvlcek.bplaced.net', 
	'Internationalize content based on slug/URL names',
	'pages',
	'i18n_pages'  
);

if (!function_exists('i18n_load_texts')) {
  function i18n_load_texts($dir, $prefix) {
    global $i18n, $LANG;
    i18n_load_texts_impl($dir, $prefix, $i18n, $LANG);
  }

  function i18n_load_texts_impl($dir, $prefix, &$globali18n, $lang) { 
    $i18n = array();
    @include(GSPLUGINPATH.$dir.'/lang/'.$lang.'.php'); 
    if (count($i18n) == 0) @include(GSPLUGINPATH.$dir.'/lang/en_US.php');
    foreach ($i18n as $code => $text) $globali18n[$prefix.$code] = $text;
  }
}

# activate filter
i18n_load_texts('i18n_base','i18n/');
add_action('index-pretemplate', 'i18n_init');
add_action('edit-extras', 'i18n_base_edit'); 
add_action('pages-sidebar', 'createSideMenu', array($thisfile, $i18n['i18n/PAGES']));

# functions
function return_i18n_languages() {
  global $i18n_languages;
  if (!$i18n_languages) {
    $i18n_languages = array();
    if (isset($_GET[I18N_LANGUAGE_PARAM])) {
      $i18n_languages[] = $_GET[I18N_LANGUAGE_PARAM];
    }
    if (isset($_SESSION[I18N_LANGUAGE_KEY]) && !in_array($_SESSION[I18N_LANGUAGE_KEY],$i18n_languages)) {
      $i18n_languages[] = $_SESSION[I18N_LANGUAGE_KEY];
    }
    $httplanguages = explode(",", $_SERVER['HTTP_ACCEPT_LANGUAGE']);
    foreach ($httplanguages as $language) {
      $language = substr($language,0,2);
      if (!in_array($language,$i18n_languages)) $i18n_languages[] = $language;
    }
    $deflang = i18n_get_property('default_language', I18N_DEFAULT_LANGUAGE);
    if (!in_array($deflang, $i18n_languages)) $i18n_languages[] = $deflang;
  }
  return $i18n_languages;
}

function i18n_init() {
  global $id;
  @session_start();
  if (isset($_GET[I18N_SET_LANGUAGE_PARAM])) {
    $_SESSION[I18N_LANGUAGE_KEY] = $_GET[I18N_SET_LANGUAGE_PARAM];
  }
  $languages = return_i18n_languages();
  foreach ($languages as $language) {
    if (i18n_load($language)) return;
  }
}

function i18n_load($lang) {
  global $language;
  global $id, $title, $date, $metak, $metad, $content, $parent, $template_file, $private;
  $deflang = i18n_get_property('default_language', I18N_DEFAULT_LANGUAGE);
  if ($lang == $deflang) {
    // custom fields:
    $file = GSDATAPAGESPATH . $id . '.xml';
    $data = getXML($file);
    i18n_base_get_custom_fields($data);
    return true;
  } else {    
    $file = GSDATAPAGESPATH . $id . '_' . $lang .'.xml';
    if (file_exists($file)) {
      $data = getXML($file);
      $title = $data->title;
      $date = $data->pubDate;
      $metak = $data->meta;
      $metad = $data->metad;
      // $url = $data->url;  // URL stays the same!
      $content = $data->content;
      $parent = $data->parent;
      $template_file = $data->template;
      $private = $data->private;
      // custom fields:
      i18n_base_get_custom_fields($data);
      // language
      $language = $lang;
      return true;
    } else {
      return false;
    }
  }
}

function i18n_base_get_custom_fields($data) {
  global $customfields;
  $stdfields = array('pubDate','title','url','meta','metad','menu','menuStatus','menuOrder',
                      'template','parent','content','private');
  $customfields = array();
  if ($data) foreach ($data->children() as $child) {
    if (!in_array($child->getName(), $stdfields)) {
      $customfields[$child->getName()] = (string) $child;
    }
  }
}

if (!function_exists('get_custom_field')) {
  function get_custom_field($name) {
    global $customfields;
    echo @$customfields[$name];
  }
}

if (!function_exists('return_custom_field')) {
  function return_custom_field($name) {
    global $customfields;
    return @$customfields[$name];
  }
}

function return_i18n_page_data($slug) {
  $languages = return_i18n_languages();
  $deflang = i18n_get_property('default_language', I18N_DEFAULT_LANGUAGE);
  foreach ($languages as $language) {
    $file = GSDATAPAGESPATH . $slug . ($lang == $deflang ? '' : '_' . $lang) .'.xml';
    if (file_exists($file)) {
      return getXML($file);
    }
  }
  return null;
}

function get_i18n_content($slug, $force=false) {
  $data = null;
  if ($force) {
    $file = GSDATAPAGESPATH . $slug . '.xml';
    if (file_exists($file)) $data = getXML($file);
  } else {
    $data = return_i18n_data($slug);
  }
  if ($data) {
    $content = $data->content;
    $content = stripslashes(htmlspecialchars_decode($content, ENT_QUOTES));
    $content = exec_filter('content',$content);
    echo $content;
  }
}

function get_i18n_component($id) {
	if (file_exists(GSDATAOTHERPATH.'components.xml')) {
    $deflang = i18n_get_property('default_language', I18N_DEFAULT_LANGUAGE);
		$data = getXML(GSDATAOTHERPATH.'components.xml');
		$components = $data->item;
  	if (count($components) != 0) {
      $languages = return_i18n_languages();
      foreach ($languages as $language) {
        $slug = $id . ($language == $deflang ? '' : $language);
			  foreach ($components as $component) {
				  if ($slug == $component->slug) { 
					  eval("?>" . stripslashes(htmlspecialchars_decode($component->value, ENT_QUOTES)) . "<?php "); 
            return;
				  }
			  }
      }
		}
	}
}

function i18n_get_property($name, $default = null) {
  global $i18n_settings;
  if ($i18n_settings == null) {
    $i18n_settings = array();
    $settingsfile = GSDATAOTHERPATH . I18N_SETTINGS_FILE;
    if (file_exists($settingsfile)) {
      $data = getXML($settingsfile);
      foreach ($data->children() as $child) $i18n_settings[$child->getName()] = (string) $child;
    }
  }
  return isset($i18n_settings[$name]) ? $i18n_settings[$name] : $default;
}

function i18n_set_property($name, $value) {
  i18n_set_properties(array($name => $value));
}

function i18n_set_properties($properties) {
  global $i18n_settings;
  $settingsfile = GSDATAOTHERPATH . I18N_SETTINGS_FILE;
	$data = @new SimpleXMLExtended('<?xml version="1.0" encoding="UTF-8"?><settings></settings>');
  foreach ($properties as $key => $value) {
    if ($value != null) {
      $propdata = $data->addChild($key);
      $propdata->addCData((string) $value);
    }
  }
  if (file_exists($settingsfile)) {
    $olddata = getXML($settingsfile);
    foreach ($olddata->children() as $child) {
      if (!isset($properties[$child->getName()])) {
        $propdata = $data->addChild($child->getName());
        $propdata->addCData((string) $child);
      }
    }
  }
	XMLsave($data, $settingsfile);
  $i18n_settings = null;
}

function i18n_pages() {
  include(GSPLUGINPATH.'i18n_base/pages.php');
}

function i18n_base_edit() {
  include(GSPLUGINPATH.'i18n_base/editextras.php');
}


